<?php
/**
 * @file
 * PWA administration forms.
 */


/**
 * Configure PWA settings for Web Push notifications.
 */
function pwa_webpush_admin_configuration() {
  $form = [];

  $form['pwa_webpush_autoregister'] = [
    '#type' => 'checkbox',
    '#title' => t('Autoregister Push notifications'),
    '#description' => t('For users who granted the notification permission, we can autoregister them to push notifications. This helps to keep users subscribed when the serviceworker changes.'),
    '#default_value' => (bool) variable_get('pwa_webpush_autoregister', TRUE),
  ];

  $form['vapid'] = [
    '#type' => 'fieldset',
    '#title' => t('Voluntary Application Server Identification'),
    '#description' => t('Manage <a href="@vapid">VAPID</a> configuration for Web Push.', ['@vapid' => 'https://tools.ietf.org/html/draft-ietf-webpush-vapid-01']),
    '#collapsible' => TRUE,
    '#collapsed' => !empty(variable_get('pwa_webpush_vapid_public', '')) && !empty(variable_get('pwa_webpush_vapid_private', '')),
  ];

  $form['vapid']['pwa_webpush_vapid_subject'] = [
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#required' => TRUE,
    '#description' => t('Optional, Contact details, can be a mailto: or https: URI.'),
    '#default_value' => variable_get('pwa_webpush_vapid_subject', 'mailto:' . variable_get('site_mail', '')),
  ];

  $form['vapid']['pwa_webpush_vapid_public'] = [
    '#type' => 'textfield',
    '#title' => t('Public key'),
    // 87 char string
    '#size' => 120,
    '#required' => TRUE,
    '#description' => t('Base64 encoded private key.'),
    // If the key is already set it can not be changed in the UI.
    '#disabled' => !empty(variable_get('pwa_webpush_vapid_public', '')),
    '#default_value' => variable_get('pwa_webpush_vapid_public', ''),
  ];

  $form['vapid']['pwa_webpush_vapid_private'] = [
    '#type' => 'textfield',
    '#title' => t('Private key'),
    // 43 char string
    '#size' => 60,
    '#required' => TRUE,
    '#description' => t('Base64 encoded private key.'),
    // If the key is already set it can not be changed in the UI.
    '#disabled' => !empty(variable_get('pwa_webpush_vapid_private', '')),
    '#default_value' => variable_get('pwa_webpush_vapid_private', ''),
  ];

  $form = system_settings_form($form);
  return $form;
}

/**
 * @param $form
 * @param $form_state
 */
function pwa_webpush_send_notification($form, &$form_state) {
  // Only users that have an active subscription.
  $uids = db_query('SELECT DISTINCT(uid) FROM {pwa_webpush_subscription} WHERE expired = 0')->fetchCol();
  $users = user_load_multiple($uids);
  $form['users'] = [
    '#type' => 'select',
    '#title' => t('Send notification to users'),
    '#description' => t('Only users who have an active subscriptions are listed.'),
    '#required' => TRUE,
    '#multiple' => TRUE,
    '#options' => array_map(function ($u) {
      return $u->uid == 0 ? 'anonymous' : $u->name;
    }, $users),
  ];

  $form['notification'] = [
    '#type' => 'fieldset',
    '#title' => t('Notification')
  ];
  $form['notification']['title'] = [
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#required' => TRUE,
    '#description' => '',
    '#default_value' => '',
  ];
  $form['notification']['body'] = [
    '#type' => 'textarea',
    '#title' => t('Body (plain text)'),
    '#required' => TRUE,
    '#description' => '',
    '#default_value' => '',
  ];
  $form['notification']['url'] = [
    '#type' => 'textfield',
    '#title' => t('URL'),
    '#description' => t('Destination when clicking on the notification.'),
    '#default_value' => url('<front>', ['absolute' => TRUE]),
  ];

  $form['metadata'] = [
    '#type' => 'fieldset',
    '#title' => t('Metadata')
  ];

  // Get icons from the manifest file.
  $icons = [];
  foreach (_pwa_manifest_file()['icons'] as $icon) {
    $icons[$icon['src']] = basename($icon['src']) . ' (' . $icon['sizes'] . ')';
  }
  $form['metadata']['icon'] = [
    '#type' => 'select',
    '#title' => t('Notification icon'),
    '#description' => '',
    '#options' => $icons,
  ];
  $form['metadata']['tag'] = [
    '#type' => 'textfield',
    '#title' => t('Tag'),
    '#description' => t('Used to group and manage notifications on the client side.'),
    '#default_value' => '',
  ];

  $form['actions'] = [];
  $form['actions']['submit'] = [
    '#type' => 'submit',
    '#value' => t('Send notification'),
  ];
  return $form;
}

/**
 * @param $form
 * @param $form_state
 */
function pwa_webpush_send_notification_validate($form, &$form_state) {
  $values = $form_state['values'];
}

/**
 * @param $form
 * @param $form_state
 */
function pwa_webpush_send_notification_submit($form, &$form_state) {
  $values = $form_state['values'];

  $payload = array_filter([
    'title' => $values['title'],
    'body' => $values['body'],
    'icon' => $values['icon'],
    'tag' => $values['tag'],
    'data' => [
      'url' => $values['url'],
    ],
  ]);

  $users = array_values($values['users']);

  watchdog('webpush', 'Notification sent to users: %users. <br> <pre>@payload</pre>', [
    '%users' => implode(', ', $users),
    '@payload' => json_encode($payload, JSON_PRETTY_PRINT),
  ]);

  pwa_webpush_send($payload, $users);

}
